ljPost = new Object();

// START ljPost SECTION...

ljPost.startPost = function() {
	// Starts posting. Requests a challenge from the server.
	// In: Nothing.
	// Out: Nothing.
	this.xmlrpc = Components.classes["@mozilla.org/xml-rpc/client2;1"].createInstance(Components.interfaces.nsIXmlRpcClient);
	this.xmlrpc.init(LiveJournal.posturl);
	document.getElementById("statuswhat").value = "Requesting challenge...";
	this.xmlrpc.asyncCall(this.challengeHandler, null, "LJ.XMLRPC.getchallenge", [], 0);
}

ljPost.challengeHandler = {
	// Amazing, we have a challenge! Time to start sending the post. Which requires calling the postToServer function!
	// In: Nothing.
	// Out: Nothing.
	onResult: function(client, ctxt, result) {
		if (dsPosts.posting) {
			var results = result.QueryInterface(Components.interfaces.nsIDictionary);
			var challenge = results.getValue("challenge").QueryInterface(Components.interfaces.nsISupportsCString);
			ljPost.postToServer(challenge);
		}
	},
	onFault: function (client, ctxt, fault) {
		if (dsPosts.posting) {
			alert('XML-RPC Fault: '+fault);
			dsPosts.sendPost();
		}
	},
	onError: function (client, ctxt, status, errorMsg) {
		if (dsPosts.posting) {
			alert('Error: '+errorMsg);
			dsPosts.sendPost();
		}
	}
}

ljPost.postToServer = function(challengestring) {
	// Sends the post to the server. Hurrah!
	// In: Nothing.
	// Out: Nothing.
	this.xmlrpc.init(LiveJournal.posturl);
	var dictionary = this.xmlrpc.createType(this.xmlrpc.STRUCT, {});
	paramstrings = new Object();
	paramints = new Object();
	paramstrings.username = LiveJournal.username;
	paramstrings.auth_method = "challenge";
	paramstrings.auth_challenge = challengestring;
	paramstrings.auth_response = hexMD5(challengestring + hexMD5(LiveJournal.password));
	paramints.ver = 1;
	paramstrings.clientversion = AppName;
	// Always use \n as the line ending... Makes life much easier, especially across platforms.
	paramstrings.lineendings = "unix";
	var subjectbox = document.getElementById("subject");
	if (subjectbox.value.trim().length > 0) {
		paramstrings.subject = subjectbox.value;
	}
	dsMidas.updateSourceTab();
	paramstrings.event = document.getElementById("messagesource").value;
	switch (this.security) {
		case 0:
			paramstrings.security = "public";
			break;
		case 1:
			paramstrings.security = "private";
			break;
		case 2:
			paramstrings.security = "usemask";
			paramints.allowmask = this.allowmask;
			break;
	}
	var now = new Date();
	paramints.year = now.getYear() + 1900;
	paramints.mon = now.getMonth() + 1;
	paramints.day = now.getDate();
	paramints.hour = now.getHours();
	paramints.min = now.getMinutes();
	
	if (this.usejournal != LiveJournal.username) {
		paramstrings.usejournal = this.usejournal;
	}

	// Ugh. Props will suck.
	var propsdictionary = this.xmlrpc.createType(this.xmlrpc.STRUCT, {});
	var propstrings = new Object();
	var propbools = new Object();
	var propints = new Object();
	var moodbox = document.getElementById("moods"); 
	if ((moodbox.selectedIndex < 0) && (moodbox.label.trim().length > 0)) {
		propstrings.current_mood = moodbox.label;
	} else if (moodbox.selectedIndex >= 0) {
		propints.current_moodid = LiveJournal.moods[moodbox.selectedIndex].id;
	}
	
	var musicbox = document.getElementById("music");
	if (musicbox.value.trim().length > 0) {
		propstrings.current_music = musicbox.value.trim();
	}
	
	if (this.dontautoformat.toString() == "true") {
		propbools.opt_preformatted = true;
	}
	
	if (this.removecomments.toString() == "true") {
		propbools.opt_nocomments = true;
	}
	
	if (this.backdate.toString() == "true") {
		propbools.opt_backdated = true;
	}
	
	var pickwbox = document.getElementById("picture");
	if (pickwbox.selectedIndex > 0) {
		propstrings.picture_keyword = this.pickw;
	}
	
	// Start crunching up objects for RPC.
	this.makeRPCObjs(paramstrings, this.xmlrpc.STRING, dictionary);
	this.makeRPCObjs(paramints, this.xmlrpc.INT, dictionary);
	this.makeRPCObjs(propstrings, this.xmlrpc.STRING, propsdictionary);
	this.makeRPCObjs(propints, this.xmlrpc.INT, propsdictionary);
	this.makeRPCObjs(propbools, this.xmlrpc.BOOLEAN, propsdictionary);
	// Finish crunching up objects. 

	dictionary.setValue("props", propsdictionary);
	document.getElementById("statuswhat").value = "Posting to journal: " + this.usejournal;
	this.xmlrpc.asyncCall(this.postHandler, null, "LJ.XMLRPC.postevent", [dictionary], 1);
}

ljPost.makeRPCObjs = function(paramobj, makeobj, dict) {
	// Hmm. Identical to the makeRPCObjs function in livejournal.js. Note to self: Combine the two.
	// In: An object. Each item must be of an identical type (specified in the second parameter).
	//	They'll all be added to the dictionary specified in  the 'dict' parameter.
	// Out: Nothing, but your dict object will be populated.
	for (i in paramobj) {
		var rpcobj = this.xmlrpc.createType(makeobj, {});
		rpcobj.data = paramobj[i];
		dict.setValue(i, rpcobj);
	}
}

ljPost.postHandler = {
	// Handle my post, dammit. Runs when the server has returned a response from the actual post.
	// In: Nothing.
	// Out: Nothing.
	onResult: function(client, ctxt, result) {
		if (dsPosts.posting) {
			// Posted!
			dsPosts.resetPostParameters();
			dsPosts.sendPost();
		}
	},
	onFault: function (client, ctxt, fault) {
		if (dsPosts.posting) {
			alert('XML-RPC Fault: '+fault);
			dsPosts.sendPost();
		}
	},
	onError: function (client, ctxt, status, errorMsg) {
		if (dsPosts.posting) {
			alert('Error: '+errorMsg);
			dsPosts.sendPost();
		}
	}
}
